# Ports, Pfade, Zeitlimits
DEFAULT_HOST = "0.0.0.0"
DEFAULT_USERNAME_FILE = "~/.localchat_user"
DEFAULT_PORT = 51121
DISCOVERY_PORT = 51111
UDP_BROADCAST_PORT = 51122
BUFFER_SIZE = 4096
ENCODING = "utf-8"